@@ -1,23 +1,45 @@
+#    VenviPy - A Virtual Environment Manager for Python.
+#    Copyright (C) 2021 - Youssef Serestou - sinusphi.sq@gmail.com
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License or any
+#    later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    A copy of the GNU General Public License version 3 named LICENSE is
+#    in the root directory of VenviPy.
+#    If not, see <https://www.gnu.org/licenses/licenses.en.html#GPL>.
+
 # -*- coding: utf-8 -*-
 """
-This module provides all the necessary data.
+This module provides several necessary data.
 """
-import xmlrpc.client
+import logging
 import shutil
 import csv
 import sys
 import os
+import re
 from subprocess import Popen, PIPE
 from dataclasses import dataclass
 
+from bs4 import BeautifulSoup
+import requests
 
-__version__ = "0.3.2"
+__version__ = "0.3.4"
 
 CFG_DIR = os.path.expanduser("~/.venvipy")
 DB_FILE = os.path.expanduser("~/.venvipy/py-installs")
-ACTIVE_FILE = os.path.expanduser("~/.venvipy/active")
+ACTIVE_FILE = os.path.expanduser("~/.venvipy/selected-dir")
+PYPI_URL = "https://pypi.org/search/"
 
 
+logger = logging.getLogger(__name__)
 
 #]===========================================================================[#
 #] FIND PYTHON 3 INSTALLATIONS [#============================================[#
@@ -42,6 +64,32 @@ def to_path(bin_path, version):
     return os.path.join(bin_path, f"python{version}")
 
 
+def is_writable(target_dir):
+    """Test whether a directory is writable.
+    """
+    if os.path.exists(target_dir):
+        test_file = os.path.join(target_dir, "test_file")
+
+        try:
+            logger.debug("Testing whether filesystem is writable...")
+            with open(test_file, "w+", encoding="utf-8") as f:
+                f.write("test")
+
+            os.remove(test_file)
+            logger.debug("Filesystem is writable")
+            return True
+
+        except OSError as e:
+            logger.debug(f"Filesystem is read-only\n{e}")
+            return False
+
+    else:
+        logger.debug(f"No such file or directory: {target_dir}")
+        return False
+
+    return False
+
+
 def ensure_confdir():
     """Create `~/.venvipy` config directory.
     """
@@ -61,19 +109,16 @@ def ensure_active_file():
     """
     ensure_confdir()
     if not os.path.exists(ACTIVE_FILE):
-        with open(ACTIVE_FILE, "w+") as f:
+        with open(ACTIVE_FILE, "w+", encoding="utf-8") as f:
             f.write("")
 
 
 def get_python_version(py_path):
     """Return Python version.
     """
-    res = Popen(
-        [py_path, "-V"],
-        stdout=PIPE,
-        universal_newlines=True
-    )
-    out, _ = res.communicate()
+    with Popen([py_path, "-V"], stdout=PIPE, text="utf-8") as res:
+        out, _ = res.communicate()
+
     python_version = out.strip()
     return python_version
 
@@ -83,12 +128,14 @@ def get_python_installs(relaunching=False):
     Write the found Python versions to `py-installs`. Create
     a new database if `relaunching=True`.
     """
-    versions = ["3.9", "3.8", "3.7", "3.6", "3.5", "3.4", "3.3"]
+    versions = [
+        "3.11", "3.10", "3.9", "3.8", "3.7", "3.6", "3.5", "3.4", "3.3"
+    ]
     py_info_list = []
     ensure_confdir()
 
     if not os.path.exists(DB_FILE) or relaunching:
-        with open(DB_FILE, "w", newline="") as cf:
+        with open(DB_FILE, "w", newline="", encoding="utf-8") as cf:
             fields = ["PYTHON_VERSION", "PYTHON_PATH"]
             writer = csv.DictWriter(
                 cf,
@@ -126,7 +173,7 @@ def add_python(py_path):
     """
     ensure_dbfile()
 
-    with open(DB_FILE, "a", newline="") as cf:
+    with open(DB_FILE, "a", newline="", encoding="utf-8") as cf:
         fields = ["PYTHON_VERSION", "PYTHON_PATH"]
         writer = csv.DictWriter(
             cf,
@@ -150,9 +197,9 @@ def remove_env():
     Remove our interpreter if we're running in a virtual
     environment.
     """
-    with open(DB_FILE, "r") as f:
+    with open(DB_FILE, "r", encoding="utf-8") as f:
         lines = f.readlines()
-    with open(DB_FILE, "w") as f:
+    with open(DB_FILE, "w", encoding="utf-8") as f:
         for line in lines:
             if sys.executable not in line.strip("\n"):
                 f.write(line)
@@ -169,6 +216,7 @@ class VenvInfo:
     venv_version: str
     site_packages: str
     is_installed: str
+    venv_comment: str
 
 
 def get_venvs(path):
@@ -176,7 +224,7 @@ def get_venvs(path):
     Get the available virtual environments
     from the specified folder.
     """
-    # return an emtpty list if directory doesn"t exist
+    # return an emtpty list if directory doesn't exist
     if not os.path.isdir(path):
         return []
 
@@ -185,6 +233,8 @@ def get_venvs(path):
     for i, venv in enumerate(os.listdir(path)):
         # build path to venv directory
         valid_venv = os.path.join(path, venv)
+
+        # only look for dirs
         if not os.path.isdir(valid_venv):
             continue
 
@@ -193,30 +243,43 @@ def get_venvs(path):
         if not os.path.isfile(cfg_file):
             continue
 
+        # build path to venvipy.cfg file
+        venvipy_cfg_file = os.path.join(valid_venv, "venvipy.cfg")
+
         venv_name = os.path.basename(valid_venv)
-        venv_version = get_pyvenv_cfg(cfg_file, "version")
-        site_packages = get_pyvenv_cfg(cfg_file, "site_packages")
-        is_installed = get_pyvenv_cfg(cfg_file, "installed")
+        venv_version = get_config(cfg_file, "version")
+        site_packages = get_config(cfg_file, "site_packages")
+        is_installed = get_config(cfg_file, "installed")
+        venv_comment = get_comment(venvipy_cfg_file)
 
         venv_info = VenvInfo(
-            venv_name, venv_version, site_packages, is_installed
+            venv_name,
+            venv_version,
+            site_packages,
+            is_installed,
+            venv_comment
         )
         venv_info_list.append(venv_info)
 
     return venv_info_list[::-1]
 
 
-def get_pyvenv_cfg(cfg_file, cfg):
+def get_config(cfg_file, cfg):
     """
     Return the values as string from a `pyvenv.cfg` file.
-    Values for `cfg` can be strings: `version`, `py_path`,
-    `site_packages` or `installed`.
+    Values for `cfg` can be: `version`, `py_path`,
+    `site_packages`, `installed`, `comment`.
     """
-    with open(cfg_file, "r") as f:
+    with open(cfg_file, "r", encoding="utf-8") as f:
         lines = f.readlines()
 
+    if lines[2][13] == ".":
+        version = lines[2][10:13].strip()  # python 3.x
+    else:
+        version = lines[2][10:14].strip()  # python 3.10+
+
     version_str = to_version(lines[2][10:].strip())
-    binary_path = to_path(lines[0][7:].strip(), lines[2][10:13].strip())
+    binary_path = to_path(lines[0][7:].strip(), version)
     site_packages = lines[1][31:].strip()
 
     if cfg == "version":
@@ -234,7 +297,7 @@ def get_pyvenv_cfg(cfg_file, cfg):
 
     if cfg == "installed":
         ensure_dbfile()
-        with open(DB_FILE, newline="") as cf:
+        with open(DB_FILE, newline="", encoding="utf-8") as cf:
             reader = csv.DictReader(cf, delimiter=",")
             for info in reader:
                 if binary_path == info["PYTHON_PATH"]:
@@ -245,22 +308,33 @@ def get_pyvenv_cfg(cfg_file, cfg):
 
 
 def get_active_dir_str():
-    """Get the default venv directory string from `active` file.
+    """Return path to selected directory.
     """
     ensure_active_file()
-    with open(ACTIVE_FILE, "r") as f:
-        active_dir = f.read()
-        return active_dir
+    with open(ACTIVE_FILE, "r", encoding="utf-8") as f:
+        selected_dir = f.read()
+        return selected_dir
     return ""
 
 
-def get_active_dir():
+def get_selected_dir():
     """
-    Get the active venv directory path string from `active`
-    file and pass it to `get_venvs()`.
+    Get the selected directory path from `selected-dir`
+    file. Return `get_venvs()`.
     """
-    active_dir = get_active_dir_str()
-    return get_venvs(active_dir)
+    selected_dir = get_active_dir_str()
+    return get_venvs(selected_dir)
+
+
+def get_comment(cfg_file):
+    """Get the comment string from `venvipy_cfg` file.
+    """
+    if os.path.exists(cfg_file):
+        with open(cfg_file, "r", encoding="utf-8") as f:
+            venv_comment = f.read()
+
+        return venv_comment
+    return ""
 
 
 #]===========================================================================[#
@@ -272,25 +346,56 @@ class PackageInfo:
     """_"""
     pkg_name: str
     pkg_version: str
+    pkg_release_date: str
     pkg_summary: str
 
 
-def get_package_infos(name):
+def get_package_infos(pkg):
     """
-    Get the package"s name, version and description
-    from [PyPI](https://pypi.org/pypi).
+    Scrape package infos from [PyPI](https://pypi.org).
     """
-    client = xmlrpc.client.ServerProxy("https://pypi.org/pypi")
-    search_result = client.search({"name": name})
-
+    snippets = []
     package_info_list = []
 
-    for i, pkg in enumerate(search_result):
-        pkg_name = pkg["name"]
-        pkg_version = pkg["version"]
-        pkg_summary = pkg["summary"]
+    for page in range(1, 3):
+        params = {"q": pkg, "page": page}
+        with requests.Session() as session:
+            res = session.get(PYPI_URL, params=params)
 
-        pkg_info = PackageInfo(pkg_name, pkg_version, pkg_summary)
+        soup = BeautifulSoup(res.text, "html.parser")
+        snippets += soup.select('a[class*="snippet"]')
+
+        if not hasattr(session, "start_url"):
+            session.start_url = res.url.rsplit("&page", maxsplit=1).pop(0)
+
+    for snippet in snippets:
+        pkg_name = re.sub(
+            r"\s+",
+            " ",
+            snippet.select_one('span[class*="name"]').text.strip()
+        )
+        pkg_version = re.sub(
+            r"\s+",
+            " ",
+            snippet.select_one('span[class*="version"]').text.strip()
+        )
+        pkg_release_date = re.sub(
+            r"\s+",
+            " ",
+            snippet.select_one('span[class*="released"]').text.strip()
+        )
+        pkg_summary = re.sub(
+            r"\s+",
+            " ",
+            snippet.select_one('p[class*="description"]').text.strip()
+        )
+
+        pkg_info = PackageInfo(
+            pkg_name,
+            pkg_version,
+            pkg_release_date,
+            pkg_summary
+        )
         package_info_list.append(pkg_info)
 
     return package_info_list[::-1]
