@@ -1,3 +1,20 @@
+#    VenviPy - A Virtual Environment Manager for Python.
+#    Copyright (C) 2021 - Youssef Serestou - sinusphi.sq@gmail.com
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License or any
+#    later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    A copy of the GNU General Public License version 3 named LICENSE is
+#    in the root directory of VenviPy.
+#    If not, see <https://www.gnu.org/licenses/licenses.en.html#GPL>.
+
 # -*- coding: utf-8 -*-
 """
 This module manages all pip processes.
@@ -78,13 +95,14 @@ class PipManager(QObject):
                 options = []
 
             venv_path = os.path.join(self._venv_dir, self._venv_name)
-            pip = f"pip {command} {' '.join(options)};"
-            pipdeptree = f"pipdeptree {' '.join(options)};"
+            pip = f"pip {command} {' '.join(options)}"
+            pipdeptree = f"pipdeptree {' '.join(options)}"
+
             task = pipdeptree if command == "pipdeptree" else pip
 
             script = (
                 f"source {venv_path}/bin/activate;"
-                f"{task}"
+                f"{task};"
                 "deactivate;"
             )
             self._process.start("bash", ["-c", script])
@@ -100,14 +118,11 @@ class PipManager(QObject):
         """Show the current process state.
         """
         if state == QProcess.Starting:
-            #print("[PROCESS]: Started")
             logger.debug("Started")
         elif state == QProcess.Running:
-            #print("[PROCESS]: Running")
             logger.debug("Running")
         elif state == QProcess.NotRunning:
-            #print("[PROCESS]: Stopped")
-            logger.info("Done.")
+            logger.debug("Done.")
             self.textChanged.emit(
                 "\n\nPress [ESC] to continue..."
             )
@@ -117,7 +132,6 @@ class PipManager(QObject):
     def on_finished(self, exitCode):
         """Show exit code when finished.
         """
-        #print(f"[PROCESS]: Exit code: {exitCode}")
         logger.debug(f"Exit code: {exitCode}")
         self._process.kill()
 
@@ -127,7 +141,6 @@ class PipManager(QObject):
         """Read from `stdout` and send the output to `update_status()`.
         """
         message = self._process.readAllStandardOutput().data().decode().strip()
-        #print(f"[PIP]: {message}")
         logger.debug(message)
         self.textChanged.emit(message)
 
@@ -137,7 +150,6 @@ class PipManager(QObject):
         """Read from `stderr`, then kill the process.
         """
         message = self._process.readAllStandardError().data().decode().strip()
-        #print(f"[ERROR]: {message}")
         logger.error(message)
         self.textChanged.emit(message)
         self.failed.emit()
