@@ -1,3 +1,20 @@
+#    VenviPy - A Virtual Environment Manager for Python.
+#    Copyright (C) 2021 - Youssef Serestou - sinusphi.sq@gmail.com
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License or any
+#    later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    A copy of the GNU General Public License version 3 named LICENSE is
+#    in the root directory of VenviPy.
+#    If not, see <https://www.gnu.org/licenses/licenses.en.html#GPL>.
+
 # -*- coding: utf-8 -*-
 """
 This module contains the tables.
@@ -46,7 +63,7 @@ class BaseTable(QTableView):
 
     def get_selected_item(self):
         """
-        Get the item name of the selected row (index 0).
+        Return `str` from `name` column of the selected row.
         """
         listed_items = self.selectionModel().selectedRows()
         for index in listed_items:
@@ -54,6 +71,16 @@ class BaseTable(QTableView):
             return selected_item
 
 
+    def get_comment(self):
+        """
+        Return `str` from `comment` column of the selected row.
+        """
+        index = self.currentIndex()
+        row_index = self.selectionModel().selectedRows()
+        row_comment = index.sibling(row_index[0].row(), 4).data()
+        return row_comment
+
+
 
 class VenvTable(BaseTable):
     """List the virtual environments found.
@@ -79,6 +106,9 @@ class VenvTable(BaseTable):
         self.folder_icon = QIcon(
             self.style().standardIcon(QStyle.SP_DirOpenIcon)
         )
+        self.info_icon = QIcon(
+            self.style().standardIcon(QStyle.SP_FileDialogInfoView)
+        )
 
         self.progress_bar = ProgBarDialog()
         self.console = ConsoleDialog()
@@ -88,9 +118,13 @@ class VenvTable(BaseTable):
         # thread
         self.thread.start()
         self.m_clone_repo_worker.moveToThread(self.thread)
+
         self.m_clone_repo_worker.started.connect(self.progress_bar.exec_)
         self.m_clone_repo_worker.finished.connect(self.progress_bar.close)
-        self.m_clone_repo_worker.finished.connect(self.finish_info)
+        self.m_clone_repo_worker.finished.connect(self.vcs_finish_info)
+
+        self.m_clone_repo_worker.failed.connect(self.progress_bar.close)
+        self.m_clone_repo_worker.failed.connect(self.vcs_abort_info)
 
         # perform a proper stop using quit() and wait()
         self.thread.finished.connect(self.thread.quit)
@@ -98,29 +132,6 @@ class VenvTable(BaseTable):
 
 
     def contextMenuEvent(self, event):
-        context_menu = QMenu(self)
-
-        # pop up only if clicking on a row
-        if self.indexAt(event.pos()).isValid():
-            context_menu.popup(QCursor.pos())
-
-        # sub menus
-        details_sub_menu = QMenu(
-            "Det&ails",
-            self,
-            icon=self.info_icon
-        )
-        install_sub_menu = QMenu(
-            "&Install",
-            self,
-            icon=QIcon.fromTheme("software-install")
-        )
-        editable_sub_menu = QMenu(
-            "&Editable",
-            self,
-            icon=QIcon.fromTheme("software-install")
-        )
-
 
         #]===================================================================[#
         #] ACTIONS [#========================================================[#
@@ -133,20 +144,30 @@ class VenvTable(BaseTable):
             statusTip="Upgrade Pip to the latest version"
         )
         upgrade_pip_action.triggered.connect(
-            lambda: self.upgrade_pip(event)
+            lambda: self.upgrade_pip(event, pkg="pip")
+        )
+
+        install_wheel_action = QAction(
+            QIcon.fromTheme("system-software-update"),
+            "Install &Wheel",
+            self,
+            statusTip="Install wheel package"
+        )
+        install_wheel_action.triggered.connect(
+            lambda: self.install_wheel(event, pkg="wheel")
         )
 
         install_packages_action = QAction(
-            "Install &additional packages",
+            "Install &packages from PyPI",
             self,
-            statusTip="Install additional packages"
+            statusTip="Install packages from PyPI"
         )
         install_packages_action.triggered.connect(
-            lambda: self.add_packages(event)
+            lambda: self.install_pypi_packages(event)
         )
 
         install_requires_action = QAction(
-            "Install from &requirements",
+            "Install from a &requirements file",
             self,
             statusTip="Install packages from requirements"
         )
@@ -155,33 +176,53 @@ class VenvTable(BaseTable):
         )
 
         install_local_action = QAction(
-            "Install &local project",
+            "Install from a &local project directory",
             self,
-            statusTip="Install a local project"
+            statusTip="Install from a local stored project directory"
         )
         install_local_action.triggered.connect(
             lambda: self.install_local(event)
         )
 
-        install_vsc_action = QAction(
-            "Install from &repository",
+        install_vcs_action = QAction(
+            "Install from a &VCS project url",
             self,
-            statusTip="Install from VSC repository"
+            statusTip="Install from a VCS url"
         )
-        install_vsc_action.triggered.connect(
-            lambda: self.install_vsc(event)
+        install_vcs_action.triggered.connect(
+            lambda: self.install_vcs(event)
         )
 
         save_requires_action = QAction(
             self.save_icon,
             "Save &requirements",
             self,
-            statusTip="Write requirements to file"
+            statusTip="Generate a requirements file"
         )
         save_requires_action.triggered.connect(
             lambda: self.save_requires(event)
         )
 
+        comment_add_action = QAction(
+            self.info_icon,
+            "&Add or modify description",
+            self,
+            statusTip="Add or modify description"
+        )
+        comment_add_action.triggered.connect(
+            lambda: self.comment_add(event)
+        )
+
+        comment_remove_action = QAction(
+            self.info_icon,
+            "&Remove description",
+            self,
+            statusTip="Remove description"
+        )
+        comment_remove_action.triggered.connect(
+            lambda: self.comment_remove(event)
+        )
+
         list_packages_action = QAction(
             "&List installed packages",
             self,
@@ -221,7 +262,7 @@ class VenvTable(BaseTable):
 
         delete_venv_action = QAction(
             self.delete_icon,
-            "&Delete environment",
+            "Delete &environment",
             self,
             statusTip="Delete environment"
         )
@@ -234,14 +275,38 @@ class VenvTable(BaseTable):
         #] MENUS [#==========================================================[#
         #]===================================================================[#
 
+        context_menu = QMenu(self)
+
+        # pop up only if clicking on a row
+        if self.indexAt(event.pos()).isValid():
+            context_menu.popup(QCursor.pos())
+
+        # sub menus
+        details_sub_menu = QMenu(
+            "Det&ails",
+            self,
+            icon=self.info_icon
+        )
+        install_sub_menu = QMenu(
+            "&Install",
+            self,
+            icon=QIcon.fromTheme("software-install")
+        )
+        comment_sub_menu = QMenu(
+            "&Description",
+            self,
+            icon=self.info_icon
+        )
+
         context_menu.addAction(upgrade_pip_action)
+        context_menu.addAction(install_wheel_action)
 
         # install sub menu
         context_menu.addMenu(install_sub_menu)
         install_sub_menu.addAction(install_packages_action)
         install_sub_menu.addAction(install_requires_action)
         install_sub_menu.addAction(install_local_action)
-        install_sub_menu.addAction(install_vsc_action)
+        install_sub_menu.addAction(install_vcs_action)
 
         # details sub meun
         context_menu.addMenu(details_sub_menu)
@@ -249,6 +314,11 @@ class VenvTable(BaseTable):
         details_sub_menu.addAction(list_freeze_action)
         details_sub_menu.addAction(list_deptree_action)
 
+        # comment sub menu
+        context_menu.addMenu(comment_sub_menu)
+        comment_sub_menu.addAction(comment_add_action)
+        comment_sub_menu.addAction(comment_remove_action)
+
         context_menu.addAction(save_requires_action)
         context_menu.addAction(open_venv_dir_action)
         context_menu.addAction(delete_venv_action)
@@ -258,11 +328,12 @@ class VenvTable(BaseTable):
         """Test wether the Python version required is installed.
         """
         cfg_file = os.path.join(venv_path, "pyvenv.cfg")
-        is_installed = get_data.get_pyvenv_cfg(cfg_file, "installed")
-        version = get_data.get_pyvenv_cfg(cfg_file, "version")
-        py_path = get_data.get_pyvenv_cfg(cfg_file, "py_path")
+        is_installed = get_data.get_config(cfg_file, "installed")
+        version = get_data.get_config(cfg_file, "version")
+        py_path = get_data.get_config(cfg_file, "py_path")
         msg_txt = (
-            f"This environment requires {version} \nfrom {py_path} which is \nnot installed.\n"
+            f"This environment requires {version} \n\
+            from {py_path} which is \nnot installed.\n"
         )
 
         if is_installed == "no":
@@ -280,7 +351,7 @@ class VenvTable(BaseTable):
 
     def venv_exists(self, path):
         """
-        Test wether the directory of the selected environment actually exists.
+        Test wether the directory of the selected environment exists.
         """
         if os.path.exists(path):
             return True
@@ -293,6 +364,8 @@ class VenvTable(BaseTable):
             self
         )
         msg_box.exec_()
+
+        # refresh venv table
         self.refresh.emit()
         return False
 
@@ -316,18 +389,30 @@ class VenvTable(BaseTable):
         return False
 
 
-    def upgrade_pip(self, event):
+    def upgrade_pip(self, event, pkg):
         """Run `pip install --upgrade pip` command.
         """
+        self.install_sys_pkg(pkg)
+
+
+    def install_wheel(self, event, pkg):
+        """Run `pip install --upgrade wheel` command.
+        """
+        self.install_sys_pkg(pkg)
+
+
+    def install_sys_pkg(self, pkg):
+        """Run `pip install --upgrade <pkg>` command.
+        """
         active_dir = get_data.get_active_dir_str()
         venv = self.get_selected_item()
 
         if self.has_pip(active_dir, venv):
-            self.console.setWindowTitle("Updating Pip")
-            logger.info("Attempting to update Pip...")
+            self.console.setWindowTitle(f"Installing latest {pkg}")
+            logger.debug(f"Installing latest version of {pkg}...")
 
             self.manager = PipManager(active_dir, venv)
-            self.manager.run_pip(creator.cmds[0], [creator.opts[0], "pip"])
+            self.manager.run_pip(creator.cmds[0], [creator.opts[0], f"{pkg}"])
             self.manager.started.connect(self.console.exec_)
 
             # display the updated output
@@ -338,11 +423,37 @@ class VenvTable(BaseTable):
                 self.console.console_window.clear()
 
 
-    def add_packages(self, event):
+    def install_pypi_packages(self, event):
         """
-        Install additional packages into the selected environment.
+        Install packages from [PyPI](https://pypi.org)
+        into the selected environment.
         """
-        pass
+        active_dir = get_data.get_active_dir_str()
+        venv = self.get_selected_item()
+
+        if self.has_pip(active_dir, venv):
+            project_name, ok = QInputDialog.getText(
+                self,
+                "Specify project",
+                "Enter a PyPI project name:" + " " * 70
+            )
+
+            if project_name != "":
+                self.console.setWindowTitle(f"Installing {project_name}")
+                logger.debug("Installing specified project...")
+
+                self.manager = PipManager(active_dir, venv)
+                self.manager.run_pip(
+                    creator.cmds[0], [creator.opts[0], f"'{project_name}'"]
+                )
+                self.manager.started.connect(self.console.exec_)
+
+                # display the updated output
+                self.manager.textChanged.connect(self.console.update_status)
+
+                # clear the content on window close
+                if self.console.close:
+                    self.console.console_window.clear()
 
 
     def install_requires(self, event):
@@ -356,14 +467,14 @@ class VenvTable(BaseTable):
         if self.has_pip(active_dir, venv):
             file_name = QFileDialog.getOpenFileName(
                 self,
-                "Select a requirements"
+                "Specify a requirements file"
             )
             file_path = file_name[0]
 
             if file_path != "":
                 creator.fix_requirements(file_path)
                 self.console.setWindowTitle("Installing from requirements")
-                logger.info("Installing from requirements...")
+                logger.debug("Installing from requirements...")
 
                 self.manager = PipManager(active_dir, venv)
                 self.manager.run_pip(
@@ -380,7 +491,7 @@ class VenvTable(BaseTable):
 
 
     def install_local(self, event):
-        """Install from a local project.
+        """Install from a local project directory.
         """
         active_dir = get_data.get_active_dir_str()
         venv = self.get_selected_item()
@@ -393,25 +504,34 @@ class VenvTable(BaseTable):
             project_name = os.path.basename(project_dir)
 
             if project_dir != "":
-                self.console.setWindowTitle(f"Installing {project_name}")
-                logger.info("Installing from local project path...")
+                if get_data.is_writable(project_dir):
+                    self.console.setWindowTitle(f"Installing {project_name}")
+                    logger.debug("Installing from local project directory...")
 
-                self.manager = PipManager(active_dir, venv)
-                self.manager.run_pip(
-                    creator.cmds[0], [creator.opts[2], f"'{project_dir}'"]
-                )
-                self.manager.started.connect(self.console.exec_)
+                    self.manager = PipManager(active_dir, venv)
+                    self.manager.run_pip(
+                        creator.cmds[0], [creator.opts[3], f"'{project_dir}'"]
+                    )
+                    self.manager.started.connect(self.console.exec_)
 
-                # display the updated output
-                self.manager.textChanged.connect(self.console.update_status)
+                    # display the updated output
+                    self.manager.textChanged.connect(self.console.update_status)
 
-                # clear the content on window close
-                if self.console.close:
-                    self.console.console_window.clear()
+                    # clear the content on window close
+                    if self.console.close:
+                        self.console.console_window.clear()
+
+                else:
+                    msg_txt = (
+                        "Filesystem is read-only.\n\n"
+                        "Unable to create the '.egg-info'      \n"
+                        "folder inside the project directory.      \n\n"
+                    )
+                    QMessageBox.warning(self, "Abort", msg_txt)
 
 
-    def install_vsc(self, event):
-        """Install from a VSC repository.
+    def install_vcs(self, event):
+        """Install from VCS.
         """
         active_dir = get_data.get_active_dir_str()
         venv = self.get_selected_item()
@@ -420,39 +540,54 @@ class VenvTable(BaseTable):
         if self.has_pip(active_dir, venv):
             url, ok = QInputDialog.getText(
                 self,
-                "Specify VSC project url",
-                "Enter url to repository:" + " " * 65
+                "Specify project",
+                "Enter link to repository:" + " " * 80
             )
 
             if url != "":
-                project_name = os.path.basename(url)
-                project_url = f"git+{url}#egg={project_name}"
+                self.vcs_project_url = url[:-4] if url.endswith(".git") else url
+                self.vcs_project_name = os.path.basename(self.vcs_project_url)
+                formatted_project_url = (
+                    f"git+{self.vcs_project_url}#egg={self.vcs_project_name}"
+                )
                 cmd = (
-                    f"{venv_bin} -m pip install --no-cache-dir -e {project_url}"
+                    f"{venv_bin} -m pip {creator.cmds[0]} {formatted_project_url}"
+                )
+                self.progress_bar.setWindowTitle(f"Installing {self.vcs_project_name}")
+                self.progress_bar.status_label.setText(
+                    "Installing from repository...  (this can take a moment)"
                 )
-                self.progress_bar.setWindowTitle(f"Installing {project_name}")
-                self.progress_bar.status_label.setText("Cloning repository...")
-                logger.info(f"Installing {project_name}...")
 
                 wrapper = partial(
-                    self.m_clone_repo_worker.run_process, cmd
+                    self.m_clone_repo_worker.run_process,
+                    cmd
                 )
                 QTimer.singleShot(0, wrapper)
 
 
-    def finish_info(self):
-        """
-        Show an info message when the cloning process has finished successfully.
+    def vcs_finish_info(self):
+        """Info message if successfully installed from repository.
         """
         msg_txt = (
-            "Successfully installed package       \nfrom VSC repository.\n"
+            "Successfully installed:           \n"
+            f"{self.vcs_project_name}\n"
         )
         QMessageBox.information(self, "Done", msg_txt)
 
 
+    def vcs_abort_info(self):
+        """Error message if installing from repository failed.
+        """
+        msg_txt = (
+            "Unable to install from\n"
+            f"{self.vcs_project_url}          \n"
+        )
+        QMessageBox.warning(self, "Abort", msg_txt)
+
+
     def save_requires(self, event):
         """
-        Write the requirements of the selected environment to file.
+        Pip freeze a requirements of the selected environment.
         """
         active_dir = get_data.get_active_dir_str()
         venv = self.get_selected_item()
@@ -462,21 +597,23 @@ class VenvTable(BaseTable):
             save_file = QFileDialog.getSaveFileName(
                 self,
                 "Save requirements",
-                directory=f"{venv_dir}/requirements.txt"
+                directory=os.path.join(venv_dir, "requirements.txt")
             )
             save_path = save_file[0]
 
             if save_path != "":
-                logger.info(f"Saving '{save_path}'...")
-
                 # write 'pip freeze' output to selected file
                 self.manager = PipManager(active_dir, venv)
                 self.manager.run_pip(creator.cmds[2], [">", save_path])
+                logger.debug(f"Saved '{save_path}'")
 
                 # show an info message
                 message_txt = (f"Saved requirements in \n{save_path}")
                 QMessageBox.information(self, "Saved", message_txt)
 
+                # comment the 'pkg_resources==0.0.0' entry
+                creator.fix_requirements(save_path)
+
 
     def list_packages(self, event, style):
         """
@@ -519,7 +656,7 @@ class VenvTable(BaseTable):
         pipdeptree_binary = os.path.join(active_dir, venv, "bin", "pipdeptree")
         has_pipdeptree = os.path.exists(pipdeptree_binary)
         message_txt = (
-            "This requires the pipdeptree package\nto be installed.\n\n"
+            "This requires pipdeptree             \nto be installed.\n\n"
             "Do you want to install it?\n"
         )
 
@@ -538,7 +675,7 @@ class VenvTable(BaseTable):
                     self.progress_bar.status_label.setText(
                         "Installing pipdeptree..."
                     )
-                    logger.info("Installing pipdeptree...")
+                    logger.debug("Installing pipdeptree...")
 
                     self.manager = PipManager(active_dir, venv)
                     self.manager.run_pip(
@@ -550,6 +687,65 @@ class VenvTable(BaseTable):
                     self.list_packages(event, style)
 
 
+    def comment_add(self, event):
+        """Add / modify a comment.
+        """
+        active_dir = get_data.get_active_dir_str()
+        venv = self.get_selected_item()
+        comment_old = self.get_comment()
+        venvipy_cfg = os.path.join(active_dir, venv, "venvipy.cfg")
+
+        if not os.path.exists(venvipy_cfg):
+            comment_new, ok = QInputDialog.getText(
+                self,
+                "Enter description",
+                "\nEnter a description:" + " " * 70
+            )
+            if len(comment_new) > 0:
+                # save a new description
+                creator.save_comment(venvipy_cfg, comment_new)
+                logger.debug("Description saved")
+
+        else:
+            comment_new, ok = QInputDialog.getText(
+                self,
+                "Modify description",
+                "\nModify your description:" + " " * 70,
+                text=comment_old
+            )
+            if len(comment_new) > 0:
+                # modify an existing description
+                creator.save_comment(venvipy_cfg, comment_new)
+                logger.debug("Description updated")
+            else:
+                # keep the old description
+                creator.save_comment(venvipy_cfg, comment_old)
+
+        # refresh venv table
+        self.refresh.emit()
+
+
+    def comment_remove(self, event):
+        """Remove comment.
+        """
+        active_dir = get_data.get_active_dir_str()
+        venv = self.get_selected_item()
+        venvipy_cfg = os.path.join(active_dir, venv, "venvipy.cfg")
+
+        if os.path.exists(venvipy_cfg):
+            msg_box_warning = QMessageBox.warning(
+                self,
+                "Remove comment",
+                "Delete this comment.        \n"
+                "Are you sure?\n",
+                QMessageBox.Yes | QMessageBox.Cancel
+            )
+            if msg_box_warning == QMessageBox.Yes:
+                os.remove(venvipy_cfg)
+                logger.debug(f"Successfully deleted '{venvipy_cfg}'")
+                self.refresh.emit()
+
+
     def open_venv_dir(self, event):
         """Open the selected venv directory.
         """
@@ -573,13 +769,14 @@ class VenvTable(BaseTable):
         if self.venv_exists(venv_path):
             msg_box_critical = QMessageBox.critical(
                 self,
-                "Confirm",
-                f"Are you sure you want to delete '{venv}'?",
+                "Delete venv",
+                f"Delete '{venv}'?           \n"
+                "Are you sure?\n",
                 QMessageBox.Yes | QMessageBox.Cancel
             )
             if msg_box_critical == QMessageBox.Yes:
                 shutil.rmtree(venv_path)
-                logging.info(f"Successfully deleted '{venv_path}'")
+                logger.debug(f"Successfully deleted '{venv_path}'")
                 self.refresh.emit()
 
 
@@ -675,16 +872,17 @@ class InterpreterTable(BaseTable):
         msg_box_warning = QMessageBox.warning(
             self,
             "Confirm",
-            "Remove this item from list.     \nAre you sure?",
+            "Remove this item from list.       \n"
+            "Are you sure?\n",
             QMessageBox.Yes | QMessageBox.Cancel
         )
         if msg_box_warning == QMessageBox.Yes:
-            with open(get_data.DB_FILE, "r") as f:
+            with open(get_data.DB_FILE, "r", encoding="utf-8") as f:
                 lines = f.readlines()
-            with open(get_data.DB_FILE, "w") as f:
+            with open(get_data.DB_FILE, "w", encoding="utf-8") as f:
                 for line in lines:
                     if item not in line:
                         f.write(line)
 
-            logging.info(f"Removed '{item}' from database")
+            logger.debug(f"Removed '{item}' from database")
             self.drop_item.emit()
