@@ -1,3 +1,20 @@
+#    VenviPy - A Virtual Environment Manager for Python.
+#    Copyright (C) 2021 - Youssef Serestou - sinusphi.sq@gmail.com
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License or any
+#    later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    A copy of the GNU General Public License version 3 named LICENSE is
+#    in the root directory of VenviPy.
+#    If not, see <https://www.gnu.org/licenses/licenses.en.html#GPL>.
+
 # -*- coding: utf-8 -*-
 """
 The main module of VenviPy.
@@ -14,6 +31,7 @@ CURRENT_DIR = Path(__file__).parent
 sys.path.insert(0, str(CURRENT_DIR))
 os.chdir(CURRENT_DIR)
 
+from PyQt5 import QtCore
 from PyQt5.QtCore import Qt, QRect, QSize, pyqtSlot
 from PyQt5.QtGui import (
     QIcon,
@@ -51,9 +69,7 @@ import wizard
 from dialogs import InfoAboutVenviPy
 from tables import VenvTable, InterpreterTable
 
-
 LOG_FORMAT = "[%(levelname)s] - { %(name)s }: %(message)s"
-logging.basicConfig(level=logging.INFO, format=LOG_FORMAT)
 logger = logging.getLogger()
 
 
@@ -70,7 +86,7 @@ class MainWindow(QMainWindow):
 
     def init_ui(self):
         self.setWindowTitle("VenviPy")
-        self.resize(1100, 690)
+        self.resize(1150, 770)
         self.center()
         self.setWindowIcon(QIcon(":/img/profile.png"))
 
@@ -316,10 +332,14 @@ class MainWindow(QMainWindow):
         h_header_venv_table.setStretchLastSection(True)
 
         # set table view model
-        self.model_venv_table = QStandardItemModel(0, 3, centralwidget)
-        self.model_venv_table.setHorizontalHeaderLabels(
-            ["Venv", "Version", "Packages", "installed"]
-        )
+        self.model_venv_table = QStandardItemModel(0, 4, centralwidget)
+        self.model_venv_table.setHorizontalHeaderLabels([
+            "Venv",
+            "Version",
+            "Packages",
+            "Installed",
+            "Description"
+        ])
         self.venv_table.setModel(self.model_venv_table)
 
         # adjust column width
@@ -431,31 +451,14 @@ class MainWindow(QMainWindow):
         menu_venv.addAction(self.action_exit)
         menu_bar.addAction(menu_venv.menuAction())
 
-        #menu_extras = QMenu("&Extras", menu_bar)
-        #menu_extras.addAction(self.action_search_pypi)
-        #menu_bar.addAction(menu_extras.menuAction())
-
         menu_help = QMenu("&Help", menu_bar)
-        menu_help.addAction(self.action_about_venvipy)
         menu_help.addAction(self.action_about_qt)
-
+        menu_help.addAction(self.action_about_venvipy)
         menu_bar.addAction(menu_help.menuAction())
 
-        msg_txt = (
-            "No suitable Python installation found!\n\n"
-            "Please specify the path to a Python (>=3.3) \n"
-            "installation or click 'Continue' to go on anyway.\n\n"
-        )
-        self.msg_box = QMessageBox(
-            QMessageBox.Critical,
-            "VenviPy",
-            msg_txt, QMessageBox.NoButton,
-            self
-        )
-
         # check if any Python is installed
         if os.path.exists(get_data.DB_FILE):
-            with open(get_data.DB_FILE, "r") as f:
+            with open(get_data.DB_FILE, "r", encoding="utf-8") as f:
                 lines = f.readlines()
             if len(lines) < 2:
                 self.launching_without_python()
@@ -464,9 +467,21 @@ class MainWindow(QMainWindow):
     def launching_without_python(self):
         """If no Python was found run with features disabled.
         """
-        logger.warning("No suitable Python installation found")
+        logger.warning("No suitable Python interpreter found")
+        msg_txt = (
+            "No suitable Python interpreter found!\n\n"
+            "Please specify the path to a Python (>=3.6) \n"
+            "interpreter or click 'Continue' to go on anyway.\n\n"
+        )
+        self.msg_box = QMessageBox(
+            QMessageBox.Critical,
+            "VenviPy",
+            msg_txt, QMessageBox.NoButton,
+            self
+        )
         self.msg_box.addButton("&Select", QMessageBox.AcceptRole)
         self.msg_box.addButton("&Continue", QMessageBox.RejectRole)
+
         if self.msg_box.exec_() == QMessageBox.AcceptRole:
             # let user specify path to an interpreter
             self.add_interpreter()
@@ -518,7 +533,7 @@ class MainWindow(QMainWindow):
         """
         get_data.ensure_dbfile()
 
-        with open(get_data.DB_FILE, newline="") as cf:
+        with open(get_data.DB_FILE, newline="", encoding="utf-8") as cf:
             reader = csv.DictReader(cf, delimiter=",")
             self.model_interpreter_table.setRowCount(0)
             for info in reader:
@@ -538,16 +553,19 @@ class MainWindow(QMainWindow):
         """
         self.model_venv_table.setRowCount(0)
 
-        for info in get_data.get_active_dir():
+        for info in get_data.get_selected_dir():
             self.model_venv_table.insertRow(0)
             for i, text in enumerate((
                     info.venv_name,
                     info.venv_version,
                     info.site_packages,
-                    info.is_installed
+                    info.is_installed,
+                    info.venv_comment
             )):
                 self.model_venv_table.setItem(0, i, QStandardItem(text))
 
+        self.model_venv_table.sort(0, QtCore.Qt.AscendingOrder)
+
 
     def update_label(self):
         """
@@ -586,13 +604,11 @@ class MainWindow(QMainWindow):
             "Open a folder containing virtual environments"
         )
         self.directory_line.setText(directory)
-
-        active_file = os.path.expanduser("~/.venvipy/active")
         active_dir = self.directory_line.text()
 
         if active_dir != "":
-            if os.path.exists(active_file):
-                with open(active_file, "w") as f:
+            if os.path.exists(get_data.ACTIVE_FILE):
+                with open(get_data.ACTIVE_FILE, "w", encoding="utf-8") as f:
                     f.write(active_dir)
                 self.pop_venv_table()
                 self.update_label()
@@ -614,8 +630,8 @@ def with_args():
     argument_list = full_cmd_arguments[1:]
 
     # tell getopts() the parameters
-    short_options = "V"
-    long_options = ["version"]
+    short_options = "Vdh"
+    long_options = ["version", "debug", "help"]
 
     # use try-except to cover errors
     try:
@@ -624,16 +640,33 @@ def with_args():
         )
     except getopt.error as e:
         # print error message and return error code
-        print(str(e))
+        err_msg = str(e)
+        print(f"O{err_msg[1:]}")
         sys.exit(2)
 
     for arg, val in arguments:
+        if arg in ("-h", "--help"):
+            # show help message, then exit
+            print(
+                f"VenviPy {get_data.__version__}  "
+                "( https://github.com/sinusphi/venvipy )\n\n"
+                "    -h --help           Show this help message and exit\n"
+                "    -d --debug          Print debugging output\n"
+                "    -v --version        Print version and exit\n"
+            )
+            sys.exit(0)
+
+        if arg in ("-d", "--debug"):
+            # verbose output for debugging
+            logging.basicConfig(level=logging.DEBUG, format=LOG_FORMAT)
+
         if arg in ("-V", "--version"):
             # print version, then exit
             print(f"VenviPy {get_data.__version__}")
             sys.exit(0)
 
 
+
 def main():
     with_args()
 
