@@ -1,3 +1,20 @@
+#    VenviPy - A Virtual Environment Manager for Python.
+#    Copyright (C) 2021 - Youssef Serestou - sinusphi.sq@gmail.com
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation, either version 3 of the License or any
+#    later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    A copy of the GNU General Public License version 3 named LICENSE is
+#    in the root directory of VenviPy.
+#    If not, see <https://www.gnu.org/licenses/licenses.en.html#GPL>.
+
 # -*- coding: utf-8 -*-
 """
 This module contains the wizard for creating
@@ -73,7 +90,7 @@ class VenvWizard(QWizard):
         super().__init__()
 
         self.setWindowTitle("VenviPy - Venv Wizard")
-        self.resize(680, 510)
+        self.resize(850, 625)
         self.center()
         self.setWindowIcon(QIcon(":/img/profile.png"))
 
@@ -124,8 +141,11 @@ class VenvWizard(QWizard):
         # process the flow only if the current page is BasicSettings()
         if self.currentId() != self.basic_settings_id:
             return super().nextId()
+
+        # go to InstallPackages() page only if pip has been selected
         if self.basic_settings.with_pip_check_box.isChecked():
             return self.install_packages_id
+
         return self.final_page_id
 
 
@@ -181,9 +201,13 @@ class BasicSettings(QWizardPage):
         self.m_install_venv_worker.started.connect(self.progress_bar.exec_)
 
         # updated
-        self.m_install_venv_worker.updatePipMsg.connect(self.update_pip_msg)
-        self.m_install_venv_worker.updatePipMsg.connect(
-            lambda: logger.info("Updating pip...")
+        self.m_install_venv_worker.updating_pip.connect(self.update_pip_msg)
+        self.m_install_venv_worker.updating_pip.connect(
+            lambda: logger.debug("Updating pip...")
+        )
+        self.m_install_venv_worker.installing_wheel.connect(self.install_wheel_msg)
+        self.m_install_venv_worker.installing_wheel.connect(
+            lambda: logger.debug("Installing wheel...")
         )
 
         # finished
@@ -215,6 +239,10 @@ class BasicSettings(QWizardPage):
         self.venv_name_line = QLineEdit()
         venv_name_label.setBuddy(self.venv_name_line)
 
+        comment_label = QLabel("&Description:")
+        self.comment_line = QLineEdit()
+        comment_label.setBuddy(self.comment_line)
+
         venv_location_label = QLabel("&Location:")
         self.venv_location_line = QLineEdit()
         venv_location_label.setBuddy(self.venv_location_line)
@@ -247,13 +275,13 @@ class BasicSettings(QWizardPage):
             checked=True,
             stateChanged=self.pip_enabled
         )
+        self.with_wheel_check_box = QCheckBox(
+            "Install &Wheel",
+            checked=True
+        )
         self.site_pkgs_check_box = QCheckBox(
-            "&Make system (global) site-packages dir available to venv"
+            "&Make system (global) site-packages available to venv"
         )
-        # since we are on linux we don't really need this setting
-        #self.symlinks_check_box = QCheckBox(
-            #"Attempt to &symlink rather than copy files into venv"
-        #)
 
         # register fields
         self.registerField(
@@ -270,12 +298,30 @@ class BasicSettings(QWizardPage):
             self.interpreter_combo_box,
             "currentData"
         )
-        self.registerField("venv_name*", self.venv_name_line)
-        self.registerField("venv_location*", self.venv_location_line)
-        self.registerField("with_pip", self.with_pip_check_box)
-        self.registerField("site_pkgs", self.site_pkgs_check_box)
-        #self.registerField("symlinks", self.symlinks_check_box)
-        self.registerField("requirements", self.requirements_line)
+        self.registerField(
+            "venv_name*",
+            self.venv_name_line
+        )
+        self.registerField(
+            "venv_location*",
+            self.venv_location_line
+        )
+        self.registerField(
+            "with_pip",
+            self.with_pip_check_box
+        )
+        self.registerField(
+            "with_wheel",
+            self.with_wheel_check_box
+        )
+        self.registerField(
+            "site_pkgs",
+            self.site_pkgs_check_box
+        )
+        self.registerField(
+            "requirements",
+            self.requirements_line
+        )
 
         # grid layout
         grid_layout = QGridLayout()
@@ -286,23 +332,26 @@ class BasicSettings(QWizardPage):
         grid_layout.addWidget(venv_name_label, 1, 0, 1, 1)
         grid_layout.addWidget(self.venv_name_line, 1, 1, 1, 1)
 
-        grid_layout.addWidget(venv_location_label, 2, 0, 1, 1)
-        grid_layout.addWidget(self.venv_location_line, 2, 1, 1, 1)
-        grid_layout.addWidget(self.select_dir_button, 2, 2, 1, 1)
+        grid_layout.addWidget(comment_label, 2, 0, 1, 1)
+        grid_layout.addWidget(self.comment_line, 2, 1, 1, 1)
 
-        grid_layout.addWidget(requirements_label, 3, 0, 1, 1)
-        grid_layout.addWidget(self.requirements_line, 3, 1, 1, 1)
-        grid_layout.addWidget(self.select_file_button, 3, 2, 1, 1)
+        grid_layout.addWidget(venv_location_label, 3, 0, 1, 1)
+        grid_layout.addWidget(self.venv_location_line, 3, 1, 1, 1)
+        grid_layout.addWidget(self.select_dir_button, 3, 2, 1, 1)
 
-        grid_layout.addWidget(place_holder, 4, 0, 1, 2)
-        grid_layout.addWidget(group_box, 5, 0, 1, 3)
+        grid_layout.addWidget(requirements_label, 4, 0, 1, 1)
+        grid_layout.addWidget(self.requirements_line, 4, 1, 1, 1)
+        grid_layout.addWidget(self.select_file_button, 4, 2, 1, 1)
+
+        grid_layout.addWidget(place_holder, 5, 0, 1, 2)
+        grid_layout.addWidget(group_box, 6, 0, 1, 3)
         self.setLayout(grid_layout)
 
         # options group box
         group_box_layout = QVBoxLayout()
         group_box_layout.addWidget(self.with_pip_check_box)
+        group_box_layout.addWidget(self.with_wheel_check_box)
         group_box_layout.addWidget(self.site_pkgs_check_box)
-        #group_box_layout.addWidget(self.symlinks_check_box)
         group_box.setLayout(group_box_layout)
 
 
@@ -312,6 +361,9 @@ class BasicSettings(QWizardPage):
         next_button.disconnect()
         next_button.clicked.connect(self.execute_venv_create)
 
+        # clear comment line
+        self.comment_line.clear()
+
 
     def pop_combo_box(self):
         """Add the selected Python version to combo box.
@@ -322,7 +374,7 @@ class BasicSettings(QWizardPage):
         self.interpreter_combo_box.clear()
         self.interpreter_combo_box.addItem("---")
 
-        with open(get_data.DB_FILE, newline="") as cf:
+        with open(get_data.DB_FILE, newline="", encoding="utf-8") as cf:
             reader = csv.DictReader(cf, delimiter=",")
             for info in reader:
                 self.interpreter_combo_box.addItem(
@@ -336,11 +388,12 @@ class BasicSettings(QWizardPage):
         """
         file_name = QFileDialog.getOpenFileName(
             self,
-            "Select Python Interpreter",
+            "Select a Python interpreter",
             "/usr/local/bin",
-            "Python binary (\
-                python3.3 python3.4 python3.5 python3.6 \
-                python3.7 python3.8 python3.9 \
+            "Python 3.3+ binary (\
+                python3.3 python3.4 python3.5 \
+                python3.6 python3.7 python3.8 \
+                python3.9 python3.10 python3.11 \
             )"
         )
         bin_file = file_name[0]
@@ -357,30 +410,38 @@ class BasicSettings(QWizardPage):
     def select_dir(self):
         """Specify path where to create the virtual environment.
         """
-        folder_name = QFileDialog.getExistingDirectory()
+        folder_name = QFileDialog.getExistingDirectory(
+            self,
+            "Select venv location",
+            directory=get_data.get_active_dir_str()
+        )
         self.venv_location_line.setText(folder_name)
 
 
     def select_file(self):
         """
-        Specify the requirements file to use
-        to clone the virtual environment.
+        Specify the requirements file to install from.
         """
-        file_name = QFileDialog.getOpenFileName()
+        file_name = QFileDialog.getOpenFileName(
+            self,
+            "Select a requirements file"
+        )
         self.requirements_line.setText(file_name[0])
 
 
     def pip_enabled(self, state):
         """
-        Enable input line for specifying a requirements file
-        only if `self.with_pip_check_box` is checked, else disable it.
+        Enable or disable input line for requirements
+        and checkbox for installing wheel.
         """
         if self.with_pip_check_box.isChecked():
             self.requirements_line.setEnabled(True)
             self.select_file_button.setEnabled(True)
+            self.with_wheel_check_box.setEnabled(True)
         else:
             self.requirements_line.setEnabled(False)
             self.select_file_button.setEnabled(False)
+            self.with_wheel_check_box.setEnabled(False)
 
 
     def execute_venv_create(self):
@@ -392,8 +453,8 @@ class BasicSettings(QWizardPage):
         self.venv_name = self.field("venv_name")
         self.venv_location = self.field("venv_location")
         self.with_pip = self.field("with_pip")
+        self.with_wheel = self.field("with_wheel")
         self.site_pkgs = self.field("site_pkgs")
-        #self.symlinks = self.field("symlinks")
         self.requirements = self.field("requirements")
 
         if self.combo_box and self.venv_name and self.venv_location:
@@ -422,8 +483,8 @@ class BasicSettings(QWizardPage):
             self.venv_name,
             self.venv_location,
             self.with_pip,
-            self.site_pkgs,
-            #self.symlinks
+            self.with_wheel,
+            self.site_pkgs
         )
 
         wrapper = partial(self.m_install_venv_worker.install_venv, args)
@@ -437,23 +498,50 @@ class BasicSettings(QWizardPage):
         self.progress_bar.status_label.setText("Updating Pip...")
 
 
+    def install_wheel_msg(self):
+        """
+        Set the text in status label to show that Pip is being updated
+        and Wheel is being installed.
+        """
+        self.progress_bar.status_label.setText(
+            "Updating Pip and installing Wheel..."
+        )
+
+
     def finish_info(self):
         """
-        Show info message when the creation process has finished successfully.
+        Save the description text and show an info message on finish.
         """
+        comment = self.comment_line.text()
+        venvipy_cfg = os.path.join(
+            self.venv_location,
+            self.venv_name,
+            "venvipy.cfg"
+        )
+        creator.save_comment(venvipy_cfg, comment)
+
+        cfg_file = os.path.join(
+            self.venv_location, self.venv_name, "pyvenv.cfg"
+        )
+        binary_path = os.path.join(self.venv_location, self.venv_name, "bin")
+        version = get_data.get_config(cfg_file, cfg="version")
         default_msg = (
             f"Virtual environment created \nsuccessfully. \n\n"
-            f"New Python {self.python_version[7:10]} executable in \n"
-            f"'{self.venv_location}/{self.venv_name}/bin'. \n"
+            f"New {version[:-2]} executable in \n"
+            f"'{binary_path}'.         \n"
         )
         with_pip_msg = ("Installed Pip and Setuptools.\n")
+        with_wheel_msg = ("Installed Pip, Setuptools and Wheel.\n")
 
-        if self.with_pip_check_box.isChecked():
+        if self.with_wheel_check_box.isChecked():
+            msg_txt = default_msg + with_wheel_msg
+        elif self.with_pip_check_box.isChecked():
             msg_txt = default_msg + with_pip_msg
         else:
             msg_txt = default_msg
 
         QMessageBox.information(self, "Done", msg_txt)
+        self.wizard().refresh.emit()  # refresh venv table in main menu
         self.wizard().next()
         self.setEnabled(True)
 
@@ -469,8 +557,9 @@ class InstallPackages(QWizardPage):
         self.setTitle("Install Packages")
         self.setSubTitle(
             "Specify the packages you want to install into the virtual "
-            "environment. Right-click on the item you want to install. "
-            "You can install as many packages as you need. When finished "
+            "environment. For more options right-click on the item you "
+            "want to install. "
+            "You can install multiple packages. When finished "
             "click next."
         )
 
@@ -501,21 +590,17 @@ class InstallPackages(QWizardPage):
             context_triggered=self.install_package  # signal
         )
 
-        # adjust vertical headers
-        v_header = self.results_table.verticalHeader()
-        v_header.setDefaultSectionSize(28)
-        v_header.hide()
+        # hide vertical header
+        self.results_table.verticalHeader().hide()
 
         # adjust horizontal headers
         h_header = self.results_table.horizontalHeader()
         h_header.setDefaultAlignment(Qt.AlignLeft)
-        h_header.setDefaultSectionSize(120)
         h_header.setStretchLastSection(True)
-        h_header.setSectionResizeMode(QHeaderView.ResizeToContents)
 
-        # item model
-        self.results_model = QStandardItemModel(0, 2, self)
-        self.results_table.setModel(self.results_model)
+        # set table view model
+        self.results_table_model = QStandardItemModel(0, 3, self)
+        self.results_table.setModel(self.results_table_model)
 
         grid_layout.addWidget(pkg_name_label, 0, 0, 1, 1)
         grid_layout.addWidget(self.pkg_name_line, 0, 1, 1, 1)
@@ -530,15 +615,26 @@ class InstallPackages(QWizardPage):
         self.venv_location = self.field("venv_location")
         self.requirements = self.field("requirements")
 
+        # run the installer if self.requirements holds a str
+        if len(self.requirements) > 0:
+            try:
+                creator.fix_requirements(self.requirements)
+            except FileNotFoundError:
+                pass  # the gui will show an error message
+            self.install_requirements()
+
         # clear all inputs and contents
-        self.results_model.clear()
+        self.results_table_model.clear()
         self.pkg_name_line.clear()
         self.pkg_name_line.setFocus(True)
 
         # set text in column headers
-        self.results_model.setHorizontalHeaderLabels(
-            ["Name", "Version", "Description"]
-        )
+        self.results_table_model.setHorizontalHeaderLabels([
+            "Name",
+            "Version",
+            "Release",
+            "Description"
+        ])
 
         # remove focus from 'next' button
         QTimer.singleShot(0, lambda: self.next_button.setDefault(False))
@@ -556,29 +652,20 @@ class InstallPackages(QWizardPage):
             self.next_button.disconnect()
             self.next_button.clicked.connect(self.save_requirements)
 
-        # run the installer if self.requirements holds a str
-        if len(self.requirements) > 0:
-            try:
-                creator.fix_requirements(self.requirements)
-            except FileNotFoundError:
-                pass  # the gui will show an error message
-            self.install_requirements()
-
 
     def install_requirements(self):
         """Install the packages from the specified requirements file.
         """
         self.setEnabled(False)
 
-        self.console.setWindowTitle("Creating environment")
-        logger.info("Creating environment...")
+        self.console.setWindowTitle("Installing packages")
+        logger.debug("Installing from requirements...")
 
         # open the console when recieving signal from manager
         self.manager = PipManager(self.venv_location, f"'{self.venv_name}'")
         self.manager.started.connect(self.console.exec_)
 
         # start installing packages from requirements file
-        #print(f"[PROCESS]: Installing packages from '{self.requirements}'")
         self.manager.run_pip(
             creator.cmds[0], [creator.opts[1], f"'{self.requirements}'"]
         )
@@ -600,32 +687,39 @@ class InstallPackages(QWizardPage):
     def pop_results_table(self):
         """Refresh the results table.
         """
+        # adjust column width
+        self.results_table.setColumnWidth(0, 200)  # name
+        self.results_table.setColumnWidth(1, 80)  # version
+        self.results_table.setColumnWidth(2, 110)  # release date
+
+        self.results_table_model.setRowCount(0)
         search_item = self.pkg_name_line.text()
 
-        self.results_model.setRowCount(0)
+        if len(search_item) >= 1:
+            for info in get_data.get_package_infos(search_item):
+                self.results_table_model.insertRow(0)
 
-        for info in get_data.get_package_infos(search_item):
-            self.results_model.insertRow(0)
+                for i, text in enumerate((
+                    info.pkg_name,
+                    info.pkg_version,
+                    info.pkg_release_date,
+                    info.pkg_summary
+                )):
+                    self.results_table_model.setItem(0, i, QStandardItem(text))
 
-            for i, text in enumerate(
-                    (info.pkg_name, info.pkg_version, info.pkg_summary)
-                ):
-                self.results_model.setItem(0, i, QStandardItem(text))
-
-        if not get_data.get_package_infos(search_item):
-            logger.debug(f"No matches for '{search_item}'")
-            QMessageBox.information(
-                self,
-                "No result",
-                f"No result matching '{search_item}'.\n"
-            )
+            if not get_data.get_package_infos(search_item):
+                logger.debug(f"No matches for '{search_item}'")
+                QMessageBox.information(
+                    self,
+                    "No results",
+                    f"No results matching '{search_item}'.\n"
+                )
 
 
     def install_package(self):
         """
-        Get the name of the selected item from the results table. Ask user
-        for confirmation before installing. If user confirmes, install the
-        selected package into the created virtual environment, else abort.
+        Get the name of the selected item from the results table. Then install
+        the selected package into the created virtual environment.
         """
         indexes = self.results_table.selectionModel().selectedRows()
         for index in sorted(indexes):
@@ -647,7 +741,7 @@ class InstallPackages(QWizardPage):
             self.manager.started.connect(self.console.exec_)
 
             # start installing the selected package
-            logger.info(f"Installing '{self.pkg}'...")
+            logger.debug(f"Installing '{self.pkg}'...")
             self.manager.run_pip(creator.cmds[0], [creator.opts[0], self.pkg])
 
             # display the updated output
@@ -664,36 +758,46 @@ class InstallPackages(QWizardPage):
 
     def save_requirements(self):
         """
-        Ask if they want to save the requirements of the
+        Ask if user want to save a requirements of the
         created virtual environment.
         """
         self.setEnabled(False)
 
-        msg_box_question = QMessageBox.question(
-            self,
+        self.msg_box = QMessageBox(
+            QMessageBox.Question,
             "Save requirements",
-            "Do you want to generate a requirements?",
-            QMessageBox.Yes | QMessageBox.No
+            "Do you want to generate a requirements?          ",
+            QMessageBox.NoButton,
+            self
         )
+        yes_button = self.msg_box.addButton("&Yes", QMessageBox.YesRole)
+        no_button = self.msg_box.addButton("&No", QMessageBox.NoRole)
+        cancel_button = self.msg_box.addButton("&Cancel", QMessageBox.RejectRole)
 
-        if msg_box_question == QMessageBox.Yes:
+        self.msg_box.exec_()
+
+        if self.msg_box.clickedButton() == yes_button:
             venv_dir = os.path.join(self.venv_location, self.venv_name)
             save_file = QFileDialog.getSaveFileName(
                 self,
                 "Save requirements",
-                directory=f"{venv_dir}/requirements.txt"
+                directory=os.path.join(venv_dir, "requirements.txt")
             )
             save_path = save_file[0]
 
-            if save_path != "":
+            if len(save_path) >= 1:
                 self.manager = PipManager(self.venv_location, self.venv_name)
                 self.manager.run_pip(creator.cmds[2], [">", save_path])
+                logger.debug(f"Saved '{save_path}'")
 
-                msg_txt = (f"Saved requirements in: \n{save_path}")
-                QMessageBox.information(self, "Saved", msg_txt)
-                logger.info(f"Saved '{save_path}'...")
+                QMessageBox.information(
+                    self,
+                    "Saved",
+                    f"Saved requirements in: \n{save_path}"
+                )
                 self.wizard().next()
-        else:
+
+        elif self.msg_box.clickedButton() == no_button:
             self.wizard().next()
 
         self.setEnabled(True)
@@ -710,8 +814,8 @@ class FinalPage(QWizardPage):
 
         self.setTitle("Finished")
         self.setSubTitle(
-            "All Tasks have been completed successfully. Click Finish "
-            "to close the wizard and return back to the main menu."
+            "All tasks have been completed successfully. Click Finish "
+            "to close the wizard."
         )
 
         h_layout = QHBoxLayout(self)
@@ -772,7 +876,7 @@ class FinalPage(QWizardPage):
         if __name__ == "__main__":
             finish_button.clicked.connect(self.wizard().force_exit)
 
-        # call update_zen_line to get a different line every new session
+        # call update_zen_line() to get a different line on every new session
         self.wizard().refresh.connect(self.update_zen_line)
         self.wizard().refresh.emit()
 
